@@ -16,7 +16,7 @@ import parqueryd
 import parqueryd.config
 from parqueryd.messages import msg_factory, Message, WorkerRegisterMessage, ErrorMessage, \
     BusyMessage, DoneMessage, StopMessage, TicketDoneMessage
-from parqueryd.tool import ens_bytes
+from parqueryd.tool import ens_bytes, ens_unicode
 from parqueryd.util import get_my_ip, bind_to_random_port
 
 POLLING_TIMEOUT = 500  # timeout in ms : how long to wait for network poll, this also affects frequency of seeing new nodes
@@ -47,7 +47,7 @@ class ControllerNode(object):
         with open(os.path.join(RUNFILES_LOCATION, 'parqueryd_controller.pid'), 'w') as F:
             F.write(str(os.getpid()))
 
-        self.logger = parqueryd.logger.getChild('controller').getChild(str(self.address))
+        self.logger = parqueryd.logger.getChild('controller').getChild(ens_unicode(self.address))
         self.logger.setLevel(loglevel)
 
         self.msg_count_in = 0
@@ -148,11 +148,12 @@ class ControllerNode(object):
     def process_sink_results(self):
         while self.rpc_results:
             msg = self.rpc_results.pop()
+
             # If this was a message to be combined, there should be a parent_token
             if 'parent_token' in msg:
                 parent_token = msg['parent_token']
                 if parent_token not in self.rpc_segments:
-                    logging.debug('Orphaned msg segment %s probabably an error was raised elsewhere' % parent_token)
+                    logging.debug('Orphaned msg segment %s probably an error was raised elsewhere' % parent_token)
                     continue
                 original_rpc = self.rpc_segments.get(parent_token)
 
@@ -165,7 +166,10 @@ class ControllerNode(object):
                         # Send the exception on to the calling RPC
                         msg['token'] = parent_token
                         msg_id = binascii.unhexlify(parent_token)
-                        self.send(msg_id, msg.to_json(), is_rpc=True)
+                        try:
+                            self.send(msg_id, msg.to_json(), is_rpc=True)
+                        except TypeError:
+                            self.logger.debug('Bytes error with ' + str(msg))
                     # and continue the processing
                     continue
 
@@ -178,6 +182,10 @@ class ControllerNode(object):
                     result_table = None
 
                 original_rpc['results'][filename] = result_table
+                logging.debug('Received result for token ' + ens_unicode(parent_token) +
+                              ' for file ' + ens_unicode(filename) + ' (' +
+                              str(len(original_rpc['results'])) + '/' + str(len(original_rpc['filenames']))
+                              + ')')
 
                 if len(original_rpc['results']) == len(original_rpc['filenames']):
                     # Check to see that there are no filenames with no Result yet
@@ -190,7 +198,16 @@ class ControllerNode(object):
                     # if finished, aggregate the result to a combined arrow table
                     pa_tables = [result_table for result_table in original_rpc['results'].values() if
                                  result_table is not None]
-                    if pa_tables:
+
+                    logging.debug('Last result received for token ' + ens_unicode(parent_token) +
+                                  ' and sending the combined result of ' +
+                                  str(len(pa_tables)) + ' relevant results')
+
+                    if len(pa_tables) == 0:
+                        msg['data'] = None
+                    elif len(pa_tables) == 1:
+                        msg['data'] = serialize_pa_table(pa_tables[0])
+                    else:
                         msg['data'] = serialize_pa_table(pa.concat_tables(pa_tables))
 
                     del self.rpc_segments[parent_token]
@@ -203,8 +220,12 @@ class ControllerNode(object):
                 self.send(msg_id, msg['data'], is_rpc=True)
                 gc.collect()
             else:
-                self.send(msg_id, msg.to_json(), is_rpc=True)
+                try:
+                    self.send(msg_id, msg.to_json(), is_rpc=True)
+                except TypeError:
+                    self.logger.debug('Bytes error with ' + str(msg))
             self.logger.debug('RPC Msg handled: %s' % msg.get('payload', '?'))
+            self.logger.debug('Pending results: %s' % str(len(self.rpc_segments)))
 
     def handle_out(self):
         # If there have been new affinity keys added, rotate them
@@ -251,7 +272,10 @@ class ControllerNode(object):
         # TODO Add a tracking of which requests have been sent out to the worker, and do retries with timeouts
         self.worker_map[worker_id]['last_sent'] = time.time()
         self.worker_map[worker_id]['busy'] = True
-        self.send(worker_id, msg.to_json())
+        try:
+            self.send(worker_id, msg.to_json())
+        except TypeError:
+            self.logger.debug('Bytes error with ' + str(msg))
 
     def handle_in(self):
         self.msg_count_in += 1
@@ -306,7 +330,7 @@ class ControllerNode(object):
         self.worker_map.setdefault(worker_id, {})['last_seen'] = time.time()
 
         if msg.isa(WorkerRegisterMessage):
-            self.logger.debug('Worker registered %s', worker_id)
+            # self.logger.debug('Worker registered %s', ens_unicode(worker_id))
             for filename in msg.get('data_files', []):
                 self.files_map.setdefault(filename, set()).add(worker_id)
             self.worker_map[worker_id]['node'] = msg.get('node', '...')
@@ -316,12 +340,12 @@ class ControllerNode(object):
             return
 
         if msg.isa(BusyMessage):
-            self.logger.debug('Worker %s sent BusyMessage', worker_id)
+            self.logger.debug('Worker %s sent BusyMessage', ens_unicode(worker_id))
             self.worker_map[worker_id]['busy'] = True
             return
 
         if msg.isa(DoneMessage):
-            self.logger.debug('Worker %s sent DoneMessage', worker_id)
+            self.logger.debug('Worker %s sent DoneMessage', ens_unicode(worker_id))
             self.worker_map[worker_id]['busy'] = False
             return
 
@@ -347,10 +371,14 @@ class ControllerNode(object):
         if 'token' in msg:
             # A message might have been passed on to a worker for processing and needs to be returned to
             #  the relevant caller so it goes in the rpc_results list
+            dbg_msg = 'Appending to queue a result for token ' + ens_unicode(msg['token'])
+            if msg.get('parent_token'):
+                dbg_msg += ' (' + ens_unicode(msg['parent_token']) + ')'
+            self.logger.debug(dbg_msg)
             self.rpc_results.append(msg)
 
     def handle_rpc(self, sender, msg):
-        # RPC calls have a binary identiy set, hexlify it to make it readable and serializable
+        # RPC calls have a binary identity set, hexlify it to make it readable and serializable
         msg_id = binascii.hexlify(sender).decode()
         msg['token'] = msg_id
         # self.logger.debug('RPC received %s' % msg_id)
@@ -473,25 +501,29 @@ class ControllerNode(object):
             if filename and filename not in self.files_map:
                 return 'Sorry, filename %s was not found' % filename
 
+        rpc_segment = {
+            'msg': msg,
+            'results': {},
+            'filenames': {x: None for x in filenames}
+        }
+
         parent_token = msg['token']
-        rpc_segment = {'msg': msg_factory(msg.copy()),
-                       'results': {},
-                       'filenames': dict([(x, None) for x in filenames])}
 
         params = {}
         for filename in filenames:
-            msg['filename'] = filename
+            worker_msg = msg.copy()
+            worker_msg['parent_token'] = parent_token
+            worker_msg['filename'] = filename
             params['args'] = list(args)
             params['args'][0] = filename
             params['kwargs'] = kwargs
-            msg['params'] = params
+            worker_msg['params'] = params
 
             # Make up a new token for the message sent to the workers, and collect the responses using that id
-            msg['parent_token'] = parent_token
             new_token = binascii.hexlify(os.urandom(8)).decode()
-            msg['token'] = new_token
+            worker_msg['token'] = new_token
             rpc_segment['filenames'][filename] = new_token
-            self.worker_out_messages.setdefault(affinity, []).append(msg.copy())
+            self.worker_out_messages.setdefault(affinity, []).append(worker_msg)
 
         self.rpc_segments[parent_token] = rpc_segment
 
