@@ -1,6 +1,5 @@
 import binascii
 import datetime
-import errno
 import gc
 import glob
 import importlib
@@ -27,7 +26,7 @@ from parquery.transport import serialize_pa_table
 import parqueryd.config
 from parqueryd.messages import msg_factory, WorkerRegisterMessage, ErrorMessage, BusyMessage, StopMessage, \
     DoneMessage, TicketDoneMessage
-from parqueryd.tool import rm_file_or_dir, ens_bytes
+from parqueryd.tool import rm_file_or_dir, ens_unicode, ens_bytes
 
 DATA_FILE_EXTENSION = '.parquet'
 # timeout in ms : how long to wait for network poll, this also affects frequency of seeing new controllers and datafiles
@@ -60,7 +59,7 @@ class WorkerBase(object):
         self.check_controllers()
         self.last_wrm = 0
         self.start_time = time.time()
-        self.logger = parqueryd.logger.getChild('worker ' + str(self.worker_id))
+        self.logger = parqueryd.logger.getChild('worker ' + ens_unicode(self.worker_id))
         self.logger.setLevel(loglevel)
         self.msg_count = 0
         signal.signal(signal.SIGTERM, self.term_signal())
@@ -75,15 +74,18 @@ class WorkerBase(object):
         return _signal_handler
 
     def send(self, addr, msg):
+        addr = ens_bytes(addr)
         try:
             if 'data' in msg:
                 data = msg['data']
                 del msg['data']
+                self.logger.debug("Sending data message back to " + ens_unicode(addr))
                 self.socket.send_multipart([addr, msg.to_json(), data])
             else:
+                self.logger.debug("Sending non-data message back to " + ens_unicode(addr))
                 self.socket.send_multipart([addr, msg.to_json()])
         except zmq.ZMQError as ze:
-            self.logger.critical("Problem with %s: %s" % (str(addr), ze))
+            self.logger.critical("Problem with %s: %s" % (ens_unicode(addr), ze))
 
     def check_controllers(self):
         # Check the Redis set of controllers to see if any new ones have appeared,
@@ -150,15 +152,14 @@ class WorkerBase(object):
             self.logger.critical('Received a msg with len != 2, something seriously wrong. ')
             return
         sender, msg_buf = tmp
-        self.logger.info("Received message from sender %s", sender)
+        self.logger.info("Received message from sender %s", ens_unicode(sender))
         msg = msg_factory(msg_buf)
 
         data = self.controllers.get(sender)
         if not data:
-            self.logger.critical('Received a msg from %s - this is an unknown sender' % sender)
+            self.logger.critical('Received a msg from %s - this is an unknown sender' % ens_unicode(sender))
             return
         data['last_seen'] = time.time()
-        self.logger.debug('Received from %s', sender)
 
         # TODO Notify Controllers that we are busy, no more messages to be sent
         # The above busy notification is not perfect as other messages might be on their way already
@@ -173,7 +174,7 @@ class WorkerBase(object):
             tmp = ErrorMessage(msg)
             tmp['payload'] = traceback.format_exc()
             self.logger.exception("Unable to handle message [%s]", msg)
-        if tmp:
+        if tmp is not None:
             self.send(sender, tmp)
 
         self.send_to_all(DoneMessage())  # Send a DoneMessage to all controllers, this flags you as 'Done'. Duh
@@ -249,7 +250,10 @@ class WorkerNode(WorkerBase):
     def execute_code(self, msg):
         args, kwargs = msg.get_args_kwargs()
 
-        func_fully_qualified_name = kwargs['function'].split('.')
+        if isinstance(kwargs['function'], bytes):
+            func_fully_qualified_name = kwargs['function'].decode().split('.')
+        else:
+            func_fully_qualified_name = kwargs['function'].split('.')
         module_name = '.'.join(func_fully_qualified_name[:-1])
         func_name = func_fully_qualified_name[-1]
         func_args = kwargs.get("args", [])
@@ -290,7 +294,7 @@ class WorkerNode(WorkerBase):
 
         # create message
         if len(pa_table) == 0:
-            msg['data'] = None
+            msg['data'] = ''
         else:
             msg['data'] = serialize_pa_table(pa_table)
 
@@ -326,14 +330,20 @@ class DownloaderNode(WorkerBase):
             random.shuffle(ticket_details_items)
 
             for node_filename_slot, progress_slot in ticket_details_items:
-                tmp = node_filename_slot.split('_')
+                if isinstance(node_filename_slot, bytes):
+                    tmp = node_filename_slot.decode().split('_')
+                else:
+                    tmp = node_filename_slot.split('_')
                 if len(tmp) < 2:
                     self.logger.critical("Bogus node_filename_slot %s", node_filename_slot)
                     continue
                 node = tmp[0]
                 filename = '_'.join(tmp[1:])
 
-                tmp = progress_slot.split('_')
+                if isinstance(progress_slot, bytes):
+                    tmp = progress_slot.decode().split('_')
+                else:
+                    tmp = progress_slot.split('_')
                 if len(tmp) != 2:
                     self.logger.critical("Bogus progress_slot %s", progress_slot)
                     continue
@@ -348,7 +358,11 @@ class DownloaderNode(WorkerBase):
 
                 try:
                     # acquire a lock for this node_filename
-                    lock_key = parqueryd.config.REDIS_DOWNLOAD_LOCK_PREFIX + self.node_name + ticket + filename
+                    lock_key = ''.join([ens_unicode(x) for x in
+                                        (parqueryd.config.REDIS_DOWNLOAD_LOCK_PREFIX,
+                                         self.node_name,
+                                         ticket,
+                                         filename)])
                     lock = self.redis_server.lock(lock_key, timeout=parqueryd.config.REDIS_DOWNLOAD_LOCK_DURATION)
                     if lock.acquire(False):
                         self.download_file(ticket, filename)
@@ -365,32 +379,41 @@ class DownloaderNode(WorkerBase):
                         pass
 
     def file_downloader_progress(self, ticket, filename, progress):
-        node_filename_slot = '%s_%s' % (self.node_name, filename)
+        node_filename_slot = '%s_%s' % (ens_unicode(self.node_name), ens_unicode(filename))
         # Check to see if the progress slot exists at all, if it does not exist this ticket has been cancelled
         # by some kind of intervention, stop the download and clean up.
-        tmp = self.redis_server.hget(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ticket, node_filename_slot)
+        tmp = self.redis_server.hget(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ens_unicode(ticket), node_filename_slot)
         if not tmp:
             # Clean up the whole ticket contents from disk
-            ticket_path = os.path.join(parqueryd.config.INCOMING, ticket)
-            self.logger.debug('Now removing entire ticket %s', ticket_path)
+            ticket_path = os.path.join(parqueryd.config.INCOMING, ens_unicode(ticket))
+            self.logger.debug('Now removing entire ticket %s', ens_unicode(ticket_path))
             for filename in glob.glob(ticket_path + '*'):
-                rm_file_or_dir(os.path.join(parqueryd.config.INCOMING, filename))
-            raise Exception("Ticket %s progress slot %s not found, aborting download" % (ticket, node_filename_slot))
+                rm_file_or_dir(os.path.join(parqueryd.config.INCOMING, ens_unicode(filename)))
+            raise Exception("Ticket %s progress slot %s not found, aborting download" %
+                            (ens_unicode(ticket), ens_unicode(node_filename_slot)))
         # A progress slot contains a timestamp_filesize
-        progress_slot = '%s_%s' % (time.time(), progress)
-        self.redis_server.hset(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ticket, node_filename_slot, progress_slot)
+        progress_slot = '%s_%s' % (time.time(), ens_unicode(progress))
+        self.redis_server.hset(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ens_unicode(ticket),
+                               ens_unicode(node_filename_slot),
+                               progress_slot)
 
     def _get_transport_params(self):
         return {}
 
     def download_file(self, ticket, fileurl):
+        ticket = ens_unicode(ticket)
+        fileurl = ens_unicode(fileurl)
+
         if self.azure_conn_string:
             self._download_file_azure(ticket, fileurl)
         else:
             self._download_file_aws(ticket, fileurl)
 
     def _download_file_aws(self, ticket, fileurl):
-        tmp = fileurl.replace('s3://', '').split('/')
+        if isinstance(fileurl, bytes):
+            tmp = fileurl.replace(b's3://', b'').decode().split('/')
+        else:
+            tmp = fileurl.replace('s3://', '').split('/')
         bucket = tmp[0]
         filename = '/'.join(tmp[1:])
 
@@ -433,10 +456,10 @@ class DownloaderNode(WorkerBase):
         self.file_downloader_progress(ticket, fileurl, 'DONE')
 
     def _get_prod_name(self, file_name):
-        return os.path.join(parqueryd.config.DEFAULT_DATA_DIR, file_name)
+        return os.path.join(parqueryd.config.DEFAULT_DATA_DIR, ens_unicode(file_name))
 
     def _get_temp_name(self, ticket, file_name):
-        return os.path.join(parqueryd.config.INCOMING, ticket + '_' + file_name)
+        return os.path.join(parqueryd.config.INCOMING, ens_unicode(ticket) + '_' + ens_unicode(file_name))
 
     def _get_s3_conn(self):
         """Create a boto3 """
@@ -451,7 +474,10 @@ class DownloaderNode(WorkerBase):
         return access_key, secret_key, s3_conn
 
     def _download_file_azure(self, ticket, fileurl):
-        tmp = fileurl.replace('azure://', '').split('/')
+        if isinstance(fileurl, bytes):
+            tmp = fileurl.decode().replace('azure://', '').split('/')
+        else:
+            tmp = fileurl.replace('azure://', '').split('/')
         container_name = tmp[0]
         blob_name = tmp[1]
         incoming_file = self._get_temp_name(ticket, blob_name)
@@ -477,11 +503,13 @@ class DownloaderNode(WorkerBase):
     def remove_ticket(self, ticket):
         # Remove all Redis entries for this node and ticket
         # it can't be done per file as we don't have the bucket name from which a file was downloaded
+        ticket = ens_unicode(ticket)
         self.logger.debug('Removing ticket %s from redis', ticket)
         for node_filename_slot in self.redis_server.hgetall(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ticket):
-            if node_filename_slot.startswith(self.node_name):
-                self.logger.debug('Removing ticket_%s %s', ticket, node_filename_slot)
-                self.redis_server.hdel(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ticket, node_filename_slot)
+            if ens_unicode(node_filename_slot).startswith(ens_unicode(self.node_name)):
+                self.logger.debug('Removing ticket_%s %s', ticket, ens_unicode(node_filename_slot))
+                self.redis_server.hdel(parqueryd.config.REDIS_TICKET_KEY_PREFIX + ticket,
+                                       ens_unicode(node_filename_slot))
         tdm = TicketDoneMessage({'ticket': ticket})
         self.send_to_all(tdm)
 
@@ -490,6 +518,8 @@ class MoveparquetNode(DownloaderNode):
     workertype = 'moveparquet'
 
     def moveparquet(self, ticket):
+        ticket = ens_unicode(ticket)
+
         # A notification from the controller that all files are downloaded on all nodes,
         # the files in this ticket can be moved into place
         ticket_path = os.path.join(parqueryd.config.INCOMING, ticket + '_*')
@@ -511,7 +541,7 @@ class MoveparquetNode(DownloaderNode):
                             }
                 open(metadata_filepath, 'w').write(json.dumps(metadata, indent=2))
 
-                self.logger.debug("Moving %s %s" % (incoming_path, prod_path))
+                self.logger.debug("Moving %s to %s" % (incoming_path, prod_path))
                 shutil.move(incoming_path, prod_path)
         else:
             self.logger.debug('Doing a moveparquet for files %s which do not exist', ticket_path)
@@ -533,14 +563,20 @@ class MoveparquetNode(DownloaderNode):
             ticket_on_this_node = False
 
             for node_filename_slot, progress_slot in ticket_details_items:
-                tmp = node_filename_slot.split('_')
+                if isinstance(node_filename_slot, bytes):
+                    tmp = node_filename_slot.decode().split('_')
+                else:
+                    tmp = node_filename_slot.split('_')
                 if len(tmp) < 2:
                     self.logger.critical("Bogus node_filename_slot %s", node_filename_slot)
                     continue
                 node = tmp[0]
                 filename = '_'.join(tmp[1:])
 
-                tmp = progress_slot.split('_')
+                if isinstance(progress_slot, bytes):
+                    tmp = progress_slot.decode().split('_')
+                else:
+                    tmp = progress_slot.split('_')
                 if len(tmp) != 2:
                     self.logger.critical("Bogus progress_slot %s", progress_slot)
                     continue
